package de.fraunhofer.iem.dataprovider.kpi.strategy

import de.fraunhofer.iem.dataprovider.kpi.dto.KPIHierarchyEdgeDto

class AggregationKPICalculationStrategy : KPICalculationStrategy {
    override fun calculateKPI(children: List<KPIHierarchyEdgeDto>): Int {
        var aggregate = 0

        val emptyChildren: List<KPIHierarchyEdgeDto> = children.filter { it.to.isEmpty() }
        val notEmptyChildren = children.toMutableList()
        notEmptyChildren.removeAll(emptyChildren)

        val additionalWeightPerElement: Double = if (notEmptyChildren.isNotEmpty()) {
            val unusedWeight = emptyChildren.map { it.weight }.reduce { acc, ele -> acc + ele }
            unusedWeight / notEmptyChildren.size
        } else {
            0.0
        }

        for (child in notEmptyChildren) {
            val childValue = child.to.getValue()
            aggregate += (childValue.toFloat() * (child.weight + additionalWeightPerElement)).toInt()
        }

        return if (notEmptyChildren.isNotEmpty()) {
            aggregate / notEmptyChildren.size
        } else {
            0
        }
    }
}
